"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const d3 = require("d3");
const axes = require("./axes");
const base_1 = require("./base");
/** The base data type for adding data to an evolution plot. */
class EvolutionItem extends base_1.BasePlotItem {
    constructor() {
        super();
        this.lab = "";
        this.cve = d3.curveCatmullRom;
        this.pnt = d3.symbol().size(1).type(d3.symbolCircle);
    }
    label(lab) {
        if (lab === undefined) {
            return this.lab;
        }
        else {
            // TODO Check certain things about labels, i.e. only whitespace is space
            this.lab = lab;
            return this;
        }
    }
    curve(cve) {
        if (cve === undefined) {
            return this.cve;
        }
        else {
            this.cve = cve;
            return this;
        }
    }
    point(pnt) {
        if (pnt === undefined) {
            return this.pnt;
        }
        else {
            this.pnt = pnt;
            return this;
        }
    }
    /** Plot the data item */
    plot(group, x, y) {
        const newGroup = this.plotSetup(group)
            .classed(base_1.prefix + this.name() + "-item", true);
        newGroup.append("g").classed(base_1.prefix + "labels", true)
            .append("g").classed(base_1.prefix + "label", true)
            .append("text")
            .attr("x", x.range().reverse()[0])
            .attr("y", y(this.target()))
            .html(this.label());
        return newGroup;
    }
}
exports.EvolutionItem = EvolutionItem;
/** A line */
class Line extends EvolutionItem {
    constructor(data) {
        super();
        this.data = data;
    }
    target() {
        return this.data[this.data.length - 1][1];
    }
    name() {
        return "line";
    }
    plot(group, x, y) {
        const lineGroup = super.plot(group, x, y);
        // TODO Truncate data if it goes outside of bounds
        const path = d3.line()
            .x(d => x(d[0]))
            .y(d => y(d[1]))
            .curve(this.cve)(this.data);
        if (path === null) {
            throw new Error("path was null");
        }
        lineGroup.append("g").classed(base_1.prefix + this.name(), true)
            .append("path").attr("d", path);
        lineGroup.append("g")
            .selectAll("g").data(this.data).enter()
            .append("g")
            .attr("transform", ([px, py]) => `translate(${x(px)}, ${y(py)})`)
            .append("g").classed(base_1.prefix + "point", true)
            .append("path").attr("d", this.pnt);
        return lineGroup;
    }
}
exports.Line = Line;
/** A span representing the evolution of a range */
class Span extends EvolutionItem {
    constructor(data) {
        super();
        this.data = data;
    }
    target() {
        const [, low, high] = this.data[this.data.length - 1];
        return (low + high) / 2;
    }
    name() {
        return "span";
    }
    plot(group, x, y) {
        const spanGroup = super.plot(group, x, y);
        // TODO Truncate data if it goes outside of bounds?
        const path = d3.area()
            .x((d) => x(d[0]))
            .y0((d) => y(d[1]))
            .y1((d) => y(d[2]))
            .curve(this.cve)(this.data);
        if (path === null) {
            throw new Error("path was null");
        }
        spanGroup.append("g").classed(base_1.prefix + this.name(), true)
            .append("path").attr("d", path);
        const pointData = this.data.map(([x, y,]) => [x, y]).concat(this.data.map(([x, , y]) => [x, y]));
        spanGroup.append("g")
            .selectAll("g").data(pointData).enter()
            .append("g")
            .attr("transform", ([px, py]) => `translate(${x(px)}, ${y(py)})`)
            .append("g").classed(base_1.prefix + "point", true)
            .append("path").attr("d", this.pnt);
        return spanGroup;
    }
}
exports.Span = Span;
/** Plot an evolution */
class Evolution extends base_1.BasePlot {
    constructor(width, height) {
        super(base_1.prefix + "mathematica");
        this.wdth = width;
        this.hght = height;
        this.data = [];
        // Bounds
        this.xMin = Infinity;
        this.xMinSet = false;
        this.xMax = -Infinity;
        this.xMaxSet = false;
        this.yMin = Infinity;
        this.yMinSet = false;
        this.yMax = -Infinity;
        this.yMaxSet = false;
        this.xScale = d3.scaleLinear();
        this.yScale = d3.scaleLinear();
        this.xLabel = "";
        this.yLabel = "";
        this.xTicks = [];
        this.yTicks = [];
        this.xTickFormat = d3.format("");
        this.yTickFormat = d3.format("");
    }
    /** Add a line to the evolution */
    line(data) {
        data.forEach(([x, y]) => {
            this.xMinSet || (this.xMin = Math.min(this.xMin, x));
            this.xMaxSet || (this.xMax = Math.max(this.xMax, x));
            this.yMinSet || (this.yMin = Math.min(this.yMin, y));
            this.yMaxSet || (this.yMax = Math.max(this.yMax, y));
        });
        const line = new Line(data);
        this.data.push(line);
        return line;
    }
    /** Add a span to the evolution */
    span(data) {
        data.forEach(([x, y0, y1]) => {
            this.xMinSet || (this.xMin = Math.min(this.xMin, x));
            this.xMaxSet || (this.xMax = Math.max(this.xMax, x));
            this.yMinSet || (this.yMin = Math.min(this.yMin, y0, y1));
            this.yMaxSet || (this.yMax = Math.max(this.yMax, y0, y1));
        });
        const span = new Span(data);
        this.data.push(span);
        return span;
    }
    /** Set the plot width */
    width(width) {
        this.wdth = width;
        return this;
    }
    /** Set the plot height */
    height(height) {
        this.hght = height;
        return this;
    }
    xmin(min) {
        if (min === undefined) {
            return this.xMin;
        }
        else {
            this.xMin = min;
            this.xMinSet = true;
            return this;
        }
    }
    xmax(max) {
        if (max === undefined) {
            return this.xMax;
        }
        else {
            this.xMax = max;
            this.xMaxSet = true;
            return this;
        }
    }
    xbounds(bounds) {
        if (bounds === undefined) {
            return [this.xmin(), this.xmax()];
        }
        else {
            const [min, max] = bounds;
            return this.xmin(min).xmax(max);
        }
    }
    ymin(min) {
        if (min === undefined) {
            return this.yMin;
        }
        else {
            this.yMin = min;
            this.yMinSet = true;
            return this;
        }
    }
    ymax(max) {
        if (max === undefined) {
            return this.yMax;
        }
        else {
            this.yMax = max;
            this.yMaxSet = true;
            return this;
        }
    }
    ybounds(bounds) {
        if (bounds === undefined) {
            return [this.ymin(), this.ymax()];
        }
        else {
            const [min, max] = bounds;
            return this.ymin(min).ymax(max);
        }
    }
    /** Set properties of the x axis */
    xaxis({ label = this.xLabel, ticks = this.xTicks, format = this.xTickFormat, scale = this.xScale }) {
        this.xLabel = label;
        this.xTicks = ticks;
        this.xTickFormat = format;
        this.xScale = scale;
        return this;
    }
    /** Set properties of the y axis */
    yaxis({ label = this.yLabel, ticks = this.yTicks, format = this.yTickFormat, scale = this.yScale }) {
        this.yLabel = label;
        this.yTicks = ticks;
        this.yTickFormat = format;
        this.yScale = scale;
        return this;
    }
    name() {
        return "evolution";
    }
    /** Plot an evolution */
    plot(svgElement) {
        const svg = super.plot(svgElement);
        const x = this.xScale.range([0, this.wdth]).domain(this.xbounds());
        const y = this.yScale.range([this.hght, 0]).domain(this.ybounds());
        // axes
        const axisGroup = svg.append("g").classed(base_1.prefix + "axis", true);
        axes.xaxis(axisGroup.append("g").attr("transform", `translate(0, ${this.hght})`), x, this.xbounds().concat(this.xTicks), this.xTickFormat, this.xLabel);
        axes.yaxis(axisGroup.append("g"), y, this.ybounds().concat(this.yTicks), this.yTickFormat, this.yLabel);
        // data
        const dataGroup = svg.append("g");
        this.data.forEach(datum => datum.plot(dataGroup, x, y));
        return svg;
    }
}
exports.Evolution = Evolution;
